/* Copyright Airship and Contributors */

#import <Foundation/Foundation.h>
#import "UARemoteDataPayload+Internal.h"
#import "UARequestSession.h"
#import "UAHTTPResponse.h"

NS_ASSUME_NONNULL_BEGIN

@interface UARemoteDataResponse : UAHTTPResponse
@property (nonatomic, readonly, nullable) NSArray<NSDictionary *> *payloads;
@property (nonatomic, readonly, nullable) NSString *lastModified;
@property (nonatomic, readonly) NSURL *requestURL;

- (instancetype)initWithStatus:(NSUInteger)status
                    requestURL:(NSURL *)requestURL
                      payloads:(nullable NSArray<NSDictionary *> *)payloads
                  lastModified:(nullable NSString *)lastModified;
@end

typedef void (^UARemoteDataAPIClientCompletionHandler)(UARemoteDataResponse * _Nullable response, NSError * _Nullable error);

/**
 * The domain for NSErrors generated by the remote data client.
 */
extern NSString * const UARemoteDataAPIClientErrorDomain;

@interface UARemoteDataAPIClient : NSObject

///---------------------------------------------------------------------------------------
/// @name Internal Methods
///---------------------------------------------------------------------------------------

/**
 * Create the remote data API client.
 *
 * @param config The Airship config.
 * @return The remote data API client instance.
 */
+ (UARemoteDataAPIClient *)clientWithConfig:(UARuntimeConfig *)config;


/**
 * Create the remote data API client. Used for testing.
 *
 * @param config The Airship config.
 * @param session The UARequestSession session.
 * @return The remote data API client instance.
 */
+ (UARemoteDataAPIClient *)clientWithConfig:(UARuntimeConfig *)config
                                    session:(UARequestSession *)session;

/**
 * Refresh the remote data by calling the remote data cloud API.
 *
 * @param locale The current locale.
 * @param lastModified Optional last modified timestamp.
 * @param completionHandler The completion handler.
 * @return A UADisposable token which can be used to cancel completion handler execution.
 *
 * Note: one block and only one block will be called.
 */
- (UADisposable *)fetchRemoteDataWithLocale:(NSLocale *)locale
                               lastModified:(nullable NSString *)lastModified
                          completionHandler:(UARemoteDataAPIClientCompletionHandler)completionHandler;

/**
 * Creaates the remote-data URL.
 * @param locale The locale.
 * @return The remote-data URL.
 */
- (NSURL *)remoteDataURLWithLocale:(NSLocale *)locale;

@end

NS_ASSUME_NONNULL_END

